if (MCE!=undefined && MCE.applicationName=='Converter') {
MCE.prefs={
	currencyControlsInitialized: false,

	// Describes the schema
	pref_list:{
		pref_24h:'boolean',
		pref_fullpage_icon:'boolean',
		pref_fullpage_menu:'boolean',
		pref_selection_menu:'boolean',
		pref_metric:'boolean',
		pref_celsius:'boolean',
		pref_in_fract:'boolean',
		pref_custom_single:'boolean',
		pref_currency_enabled:'boolean',
		pref_myCurrency:'string',
		pref_custom_convert_selection:'boolean'
	},
	
	// Describes the defaults
	pref_defaults:{
		pref_24h:true,
		pref_fullpage_icon:true,
		pref_fullpage_menu:false,
		pref_selection_menu:true,
		pref_metric:true,
		pref_celsius:true,
		pref_in_fract:true,
		pref_custom_single:true,
		pref_currency_enabled:true,
		pref_myCurrency:'',
		pref_custom_convert_selection:true
	},
	
	// Contains the cached prefs
	prefs:{
	},

	// Have we cached the prefs?
	prefs_cached:false,
	
	// Our very own nsIPrefBranch object; it gets destroyed after each operation
	pO:false,
	
	// public; direct use encouraged
	getPref:function(pref)
	{
		this.buildCache();
		return this.prefs[pref];
	},
	
	// public; direct use encouraged
	setPref:function(pref,val)
	{
		if (!this._writePref(pref,val)) {
			return false;
		}
		this.prefs[pref]=val;
		return true;
	},
	
	// public; direct use discouraged
	buildCache:function()
	{
		if (this.prefs_cached) {
			return true;
		}
		var prefs=this.pO;
		for(var pref in this.pref_list) {
			if (prefs.getPrefType(pref)==prefs.PREF_INVALID) {
				this.setPref(pref,this.pref_defaults[pref]);
			}
			this.prefs[pref]=this._readPref(pref);
		}
		return true;
	},
	
	// public; direct use discouraged
	clearCache:function()
	{
		this.prefs_cached=false;
	},

	init:function()
	{
		var prefs = Components.classes["@mozilla.org/preferences-service;1"].
		getService(Components.interfaces.nsIPrefService);
		prefs = prefs.getBranch("extensions.converter.preferences.");
		this.pO=prefs;
	},
	
	// private
	_readPref:function(pref)
	{
		if(this.pref_list[pref]=='boolean') {
			return this.pO.getBoolPref(pref);
		}
		if (this.pref_list[pref]=='string') {
			return this.pO.getCharPref(pref);
		}
		// to be extended
		return null;
	},
	
	// private
	_writePref:function(pref,val)
	{
		// Too lazy to check whether Moz prefs match exactly on JS types, so
		// I'll check manually for now.
		if(this.pref_list[pref]=='boolean') {
			if(typeof(val)!='boolean') {
				return false;
			}
			this.pO.setBoolPref(pref,val);
			return true;
		}
		if (this.pref_list[pref]=='string') {
			this.pO.setCharPref(pref,val);
			return true;
		}
		// to be extended
		return null;
	},

	prefsOnLoad:function()
	{
		document.getElementById("pref_24h").value=this.getPref('pref_24h')?'1':'0';
		document.getElementById("pref_fullpage_icon").checked=this.getPref('pref_fullpage_icon');
		document.getElementById("pref_fullpage_menu").checked=this.getPref('pref_fullpage_menu');
		document.getElementById("pref_selection_menu").checked=this.getPref('pref_selection_menu');
		document.getElementById("pref_metric").value=this.getPref('pref_metric')?'1':'0';
		document.getElementById("pref_celsius").value=this.getPref('pref_celsius')?'1':'0';
		document.getElementById("pref_in_fract").value=this.getPref('pref_in_fract')?'1':'0';
		document.getElementById("pref_custom_convert_selection").checked=this.getPref('pref_custom_convert_selection');

		// Currency
		document.getElementById("pref_currency_enabled").checked=this.getPref('pref_currency_enabled');
		this.initCurrencyControls(this.getPref('pref_currency_enabled'));

                document.addEventListener('keyup',MCE.prefs.keyPressed,false);

		return true;
	},

	keyPressed:function(e)
	{
		if (e.keyCode==27) {
			MCE.prefs.close_cancel();
		}
		if (e.keyCode==13) {
			MCE.prefs.close_ok();
		}
	},

	initCurrencyControls:function(enabled)
	{
		var list=document.getElementById("pref_myCurrency");

		document.getElementById("pref_myCurrencyLabel").disabled=!enabled;
		list.disabled=!enabled;

		if (this.currencyControlsInitialized) {
			if (list.value && list.getItemAtIndex(0).value=='') {
				list.removeItemAt(0);
			}
			return true;
		}

		this.currencyControlsInitialized=true;

		var listIndex=MCE.util.inArray(this.getPref('pref_myCurrency'),MCE.currency.currencies);
		if (listIndex==-1) {
			listIndex=0;
			list.appendItem("Please select one in order to use this feature",'');
		}
		var cc; // Currency code
		var cn; // Currency name
		for(var i=0;i<MCE.currency.currencies.length;i++) {
			cc=MCE.currency.currencies[i];
			if (cc in MCE.currency.currency_names) {
				cn=' ('+MCE.currency.currency_names[cc]+')';
			} else {
				cn='';
			}
			list.appendItem(
				cc, /* label */
				cc, /* value */
				cn /* description */
			);	
		}
		list.selectedIndex=listIndex;
		return true;
	},

	close_ok:function()
	{
		if (
			document.getElementById("pref_currency_enabled").checked &&
			!document.getElementById("pref_myCurrency").value
		) {
			alert(
				"If you want to use the currency conversion features\n"+
				"you have to select the target currency."
			);
			return false;
		}
		this.setPref("pref_24h",document.getElementById("pref_24h").value=="1");
		this.setPref("pref_fullpage_icon",document.getElementById("pref_fullpage_icon").checked);
		this.setPref("pref_fullpage_menu",document.getElementById("pref_fullpage_menu").checked);
		this.setPref("pref_selection_menu",document.getElementById("pref_selection_menu").checked);
		this.setPref("pref_metric",document.getElementById("pref_metric").value=="1");
		this.setPref("pref_celsius",document.getElementById("pref_celsius").value=="1");
		this.setPref("pref_in_fract",document.getElementById("pref_in_fract").value=="1");
		this.setPref("pref_currency_enabled",document.getElementById("pref_currency_enabled").checked);
		this.setPref("pref_myCurrency",document.getElementById("pref_myCurrency").value);
		this.setPref("pref_custom_convert_selection",document.getElementById("pref_custom_convert_selection").checked);
		this.prefActions();
		window.close();
	},

	close_cancel:function()
	{
		if (
			this.getPref("pref_currency_enabled") &&
			!this.getPref("pref_myCurrency")
		) {
			document.getElementById("pref_currency_enabled").checked=false;
			this.setPref("pref_currency_enabled",false);
			this.initCurrencyControls(false);
			
			alert(
				"Currency conversions have been disabled.\n"+
				"You can enable them at any time in the options dialog."
			);
		}
		/*
			Sometimes this is opened as modal. In that case, event listeners seem to
			be inherited by the opener. Failing to remove them causes the opener
			to misbehave (i.e. events still get captured by the modal window's listeners
			even after it gets closed, although that happens in a sick and twisted way
			which doesn't allow the listeners to actually perform; probably a bad case
			of good practice plus security concerns mangling this up).
		*/
		document.removeEventListener('keyup',MCE.prefs.keyPressed,false);
		window.close();
	},

	prefActions:function()
	{
		var myWindow = Components.classes["@mozilla.org/appshell/window-mediator;1"].getService(Components.interfaces.nsIWindowMediator).getMostRecentWindow("navigator:browser");
		myWindow.MCE.iface.initControls();
		return true;
	}
}

MCE.prefs.init();

}
